@@ -5,7 +5,7 @@
 # RTL8168C/8111C, RTL8168CP/8111CP, RTL8168D/8111D, and RTL8168DP/8111DP, and
 # RTK8168E/8111E Gigabit Ethernet controllers with PCI-Express interface.
 #
-# Copyright(c) 2011 Realtek Semiconductor Corp. All rights reserved.
+# Copyright(c) 2012 Realtek Semiconductor Corp. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -105,12 +105,12 @@
 #define NAPI_SUFFIX	""
 #endif
 
-#define RTL8168_VERSION "8.028.00" NAPI_SUFFIX
+#define RTL8168_VERSION "8.031.00" NAPI_SUFFIX
 #define MODULENAME "r8168"
 #define PFX MODULENAME ": "
 
 #define GPL_CLAIM "\
-r8168  Copyright (C) 2011  Realtek NIC software team <nicfae@realtek.com> \n \
+r8168  Copyright (C) 2012  Realtek NIC software team <nicfae@realtek.com> \n \
 This program comes with ABSOLUTELY NO WARRANTY; for details, please see <http://www.gnu.org/licenses/>. \n \
 This is free software, and you are welcome to redistribute it under certain conditions; see <http://www.gnu.org/licenses/>. \n"
 
@@ -799,6 +799,7 @@ enum RTL8168_registers {
 	Config3			= 0x54,
 	Config4			= 0x55,
 	Config5			= 0x56,
+	TDFNR			= 0x57,
 	TimeIntr		= 0x58,
 	PHYAR			= 0x60,
 	CSIDR			= 0x64,
@@ -812,8 +813,11 @@ enum RTL8168_registers {
 	EPHY_RXER_NUM		= 0x7C,
 	EPHYAR			= 0x80,
 	OCPDR			= 0xB0,
+	MACOCP			= 0xB0,
 	OCPAR			= 0xB4,
+	PHYOCP			= 0xB8,
 	DBG_reg			= 0xD1,
+	MCUCmd_reg		= 0xD3,
 	RxMaxSize		= 0xDA,
 	EFUSEAR			= 0xDC,
 	CPlusCmd		= 0xE0,
@@ -985,6 +989,7 @@ enum RTL8168_register_content {
 	ERIAR_ExGMAC = 0,
 	ERIAR_MSIX = 1,
 	ERIAR_ASF = 2,
+	ERIAR_OOB = 2,
 	ERIAR_Type_shift = 16,
 	ERIAR_ByteEn = 0x0f,
 	ERIAR_ByteEn_shift = 12,
@@ -998,6 +1003,15 @@ enum RTL8168_register_content {
 	OCPAR_Flag = 0x80000000,
 	OCPAR_GPHY_Write = 0x8000F060,
 	OCPAR_GPHY_Read = 0x0000F060,
+	OCPR_Write = 0x80000000,
+	OCPR_Read = 0x00000000,
+	OCPR_Addr_Reg_shift = 16,
+	OCPR_Flag = 0x80000000,
+	
+	/* MCU Command */
+	Now_is_oob = (1 << 7),
+	Txfifo_empty = (1 << 5),
+	Rxfifo_empty = (1 << 4),
 
 	/* E-FUSE access */
 	EFUSE_WRITE	= 0x80000000,
@@ -1200,6 +1214,7 @@ struct rtl8168_private {
 	u8	duplex;
 	u16	speed;
 	u16	eeprom_len;
+	u16	cur_page;
 
 	int (*set_speed)(struct net_device *, u8 autoneg, u16 speed, u8 duplex);
 	void (*get_settings)(struct net_device *, struct ethtool_cmd *);
@@ -1242,6 +1257,9 @@ enum mcfg {
 	CFG_METHOD_18,
 	CFG_METHOD_19,
 	CFG_METHOD_20,
+	CFG_METHOD_21,
+	CFG_METHOD_22,
+	CFG_METHOD_23,
 	CFG_METHOD_MAX,
 	CFG_METHOD_DEFAULT = 0xFF
 };
@@ -1251,16 +1269,18 @@ enum mcfg {
 #define OOB_CMD_DRIVER_STOP	0x06
 #define OOB_CMD_SET_IPMAC	0x41
 
-extern void mdio_write(struct rtl8168_private *tp, u32 RegAddr, u32 value);
-extern void rtl8168_ephy_write(void __iomem *ioaddr, int RegAddr, int value);
-extern void OCP_write(struct rtl8168_private *tp, u8 mask, u16 Reg, u32 data);
-extern void OOB_notify(struct rtl8168_private *tp, u8 cmd);
-extern void rtl8168_init_ring_indexes(struct rtl8168_private *tp);
-extern int rtl8168_eri_write(void __iomem *ioaddr, int addr, int len, u32 value, int type);
-extern u32 mdio_read(struct rtl8168_private *tp, u32 RegAddr);
-extern u32 OCP_read(struct rtl8168_private *tp, u8 mask, u16 Reg);
-extern u32 rtl8168_eri_read(void __iomem *ioaddr, int addr, int len, int type);
-extern u16 rtl8168_ephy_read(void __iomem *ioaddr, int RegAddr);
+void mdio_write(struct rtl8168_private *tp, u32 RegAddr, u32 value);
+void rtl8168_ephy_write(void __iomem *ioaddr, int RegAddr, int value);
+void OCP_write(struct rtl8168_private *tp, u8 mask, u16 Reg, u32 data);
+void OOB_notify(struct rtl8168_private *tp, u8 cmd);
+void rtl8168_init_ring_indexes(struct rtl8168_private *tp);
+int rtl8168_eri_write(void __iomem *ioaddr, int addr, int len, u32 value, int type);
+void OOB_mutex_lock(struct rtl8168_private *tp);
+u32 mdio_read(struct rtl8168_private *tp, u32 RegAddr);
+u32 OCP_read(struct rtl8168_private *tp, u8 mask, u16 Reg);
+u32 rtl8168_eri_read(void __iomem *ioaddr, int addr, int len, int type);
+u16 rtl8168_ephy_read(void __iomem *ioaddr, int RegAddr);
+void OOB_mutex_unlock(struct rtl8168_private *tp);
 
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
